package My::Config;
## -*-cperl-*-
## CQPDemo: basic configuration settings and global initialisation (starts CQP backend etc.)

use Carp;

use CWB;
use CWB::CL;
use CWB::CQP;
use CWB::Web::Cache;

##
##  User configuration area: edit to match your installation and preferences.
##

BEGIN {
  $BaseDir = "/CQP/Europarl";     # base URL for HTML pages and data files
  $CacheDir = "/tmp/Europarl-Cache";     # local cache directory for persistent query results
  $CacheSize = 10;                                # maximal cache size in MBytes
  $CacheExpire = 24;                              # maximal storage time before query results expire (in hours)

  $ImgDir = "$BaseDir/img";
  $CSSDir = "$BaseDir/css";

  @Languages = qw(EN DE FR ES IT NL);             # available languages (uppercase ISO codes) and ordering
  %LanguageNames = qw(EN English  DE German  FR French  ES Spanish  IT Italian  NL Dutch); # lookup hash for language names
  $DefaultLanguage = "EN";                        # default language for queries
  $CorpusBase = "EUROPARL-";                      # base name of CWB corpus (has to be extended with language code!)
  $Registry = undef;                              # set to registry directory if not in standard registry
  $RegistryFlag = (defined $Registry) ? "-r ".CWB::Shell::Quote($Registry) : ""; # registry flag to pass to CWB tools

  $MaxMatches = 50000;                            # maximum number of matches allowed in CQP queries
}

use My::Param;                                  # parse CGI parameters after basic configuration variables have been set
$Corpus = $CorpusBase . My::Param::Get("language");  # selected corpus for queries and word lists


##
##  Load and configure CGI module (should automatically apply to other modules importing CGI.pm)
##

use CGI qw/:standard/;

$CGI::POST_MAX = 50 * 1024;                     # avoid denial-of-service attacks
$CGI::DISABLE_UPLOADS = 1;                      # (no file uploads, accept max. 50k of POST data)


##
##  Start CQP process and initialise persistency cache
##


$CQP = new CWB::CQP;                            # incurs moderate overhead for CGI scripts that do not use CQP
croak "Error: Can't start CQP backend."
  unless defined $CQP;
$CQP->set_error_handler(\&CQP_Error_Handler);   # prints HTML page with error message
$CQP->exec("set Registry '$Registry'")          # set non-standard registry directory
  if defined $Registry;

$Cache = new CWB::Web::Cache -cqp => $CQP, -cachedir => $CacheDir, -cachesize => $CacheSize, -cachetime => $CacheExpire;


##
##  Error handlers (display error message as HTML page)
##

# My::Config::Error($header, $line1, ...);
#   generic error handler, prints headline and body of error message
sub Error {
  my $header = shift;
  print(
        header,
        start_html(-title => "CQPDemo: Error",
                   -style => { -src => "$CSSDir/standard.css" }),
        "\n",
        h1({-class => "error"}, escapeHTML($header)), "\n",
        pre(join("\n", map { escapeHTML($_) } @_), "\n"), "\n",
        end_html);
  exit 0;
}

# CQP error handler for shared CQP backend (calls My::Config::Error to print CQP error message)
sub CQP_Error_Handler {
  Error("Query execution error:", @_);
}


##
##  Direct corpus access (using the CL module)
##

%Attributes = ();                               # cache requested attribute handles ($Attributes{$lang}{$name})
$CWB::CL::Registry = $Registry                  # set non-standard registry directory
  if defined $Registry;
%CorpusHandle = ();             # hash of corpus handles ($CorpusHandle{$lang})
foreach $lang (@Languages) {
        $CorpusHandle{$lang} = new CWB::CL::Corpus "${CorpusBase}${lang}";
        croak "Error: can't open corpus ${CorpusBase}${lang}, aborted."
        unless defined $CorpusHandle{$lang};
}

# $att_handle = My::Config::GetAttribute($name [, $lang]);
#   - get CL attribute handle for specified attribute $name (returns CWB::CL::Attribute object)
#   - optionally specify required language (i.e. corpus); defaults to "language" parameter setting
sub GetAttribute {
  croak 'Usage:  $att_handle = My::Config::GetAttribute($name [, $lang]);'
    unless @_ == 1 or @_ == 2;
  my $name = shift;
  my $lang = (@_) ? uc(shift) : My::Param::Get("language");
  return $Attributes{$lang}{$name}                      # retrieve attribute handle from cache
    if exists $Attributes{$lang}{$name};
  my $att = $CorpusHandle{$lang}->attribute($name, "p"); # try p-attribute first ...
  $att = $CorpusHandle{$lang}->attribute($name, "s")      # ... then try s-attribute
    unless defined $att;
  $att = $CorpusHandle{$lang}->attribute($name, "a")      # ... finally try a-attribute
    unless defined $att;
  croak "Can't open attribute ${CorpusBase}${lang}.$name, sorry."
    unless defined $att;
  $Attributes{$lang}{$name} = $att;                     # store attribute handle in cache
  return $att;
}


##
##  My::Config loaded successfully
##

return 1;
