package My::KWIC;
## -*-cperl-*-
## CQPDemo: flexible kwic formatter implemented in Perl (using CWB::CL interface)

use Carp;

use My::Config;
use My::Param;

use CGI qw/:standard/;

##
##  compute context of specified region from 'context' parameter
##

# ($c_start, $c_end, $ellip_start, $ellip_end) = My::KWIC::Context($lang, $start, $end);
#   start and end cpos of context around region $start .. $end;
#   $words_start and $words_end indicate that it might be appropriate to put an ellipsis "..." before/after context
sub Context {
  croak 'Usage:  ($c_start, $c_end) = My::KWIC::Context($lang, $start, $end [, $context]);'
    unless @_ == 3 or @_ == 4;
  my $lang    = shift;
  my $s       = shift;
  my $e       = shift;
  my $context = (@_) ? shift : My::Param::Get("context");
  $context =~ /^\s*([0-9]*)\s*([a-z_][a-z0-9_-]*)\s*$/
    or croak "My::KWIC::Context: invalid context specification '$context'.";
  my $num  = $1;
  my $unit = $2;
  $num = 1
    unless $num;
  croak "My::KWIC::Context: invalid context specification '$context'."
    unless $num >= 1;
  my $cs          = $s - 5;    # default context: 5 words (if appropriate context cannot be determined)
  my $ce          = $e + 5;
  my $ellip_start = 1;
  my $ellip_end   = 1;

  if ( $unit =~ /^words?$/ ) {
    $cs = $s - $num;
    $ce = $e + $num;
  }
  else {
    my $att = My::Config::GetAttribute( $unit, $lang );    # hope it's a structural attribute, otherwise we'll just crash
    croak "My::KWIC::Context: attribute does not exist in context specification '$context'."
      unless defined $att;
    my $struc = $att->cpos2struc($s);
    if ( defined $struc ) {
      $struc = $struc + 1 - $num;
      $struc = 0
        if $struc < 0;
      ( $cs, undef ) = $att->struc2cpos($struc);
      $ellip_start = 0;
    }
    $struc = $att->cpos2struc($e);
    if ( defined $struc ) {
      $struc = $struc - 1 + $num;
      my $max = $att->max_struc;
      $struc = $max - 1
        if $struc >= $max;
      ( undef, $ce ) = $att->struc2cpos($struc);
      $ellip_end = 0;
    }
  }

  # now make sure context doesn't extend beyond corpus limits
  my $att = My::Config::GetAttribute( "word", $lang );
  my $max = $att->max_cpos;
  if ( $cs < 0 ) {
    $ellip_start = 0;
    $cs          = 0;
  }
  if ( $ce >= $max ) {
    $ellip_end = 0;
    $ce        = $max - 1;
  }
  return ( $cs, $ce, $ellip_start, $ellip_end );
}

##
##  "translate" specified region to other language
##

# ($target_start, $target_end) = My::KWIC::Translate($start, $end, $lang [, $source_lang]);
#   - returns (undef, undef) if region is not aligned to target language
#   - $source_lang defaults to current main language
sub Translate {
  croak 'Usage:  ($start, $end) = My::KWIC::Translate($start, $end, $lang [, $source_lang]);'
    unless @_ == 3 or @_ == 4;
  my $start       = shift;
  my $end         = shift;
  my $target_lang = lc(shift);
  my $lang        = (@_) ? uc(shift) : My::Param::Get("language");
  my ( $target_start, $target_end ) = ( undef, undef );

  my $att = My::Config::GetAttribute( lc($My::Config::CorpusBase) . $target_lang, $lang );
  my $start_region = $att->cpos2alg($start);

  return ( undef, undef ) unless defined $start_region;
  ( undef, undef, $target_start, undef ) = $att->alg2cpos($start_region);
  my $end_region = $att->cpos2alg($end);

  return ( undef, undef ) unless defined $end_region;
  ( undef, undef, undef, $target_end ) = $att->alg2cpos($end_region);

  return ( $target_start, $target_end );
}

##
##  format KWIC string according to relevant parameter settings
##

# $html_string = My::KWIC::Format($lang, $start, $end [, $target, $keyword]);
#   format corpus range $s .. $e as HTML string according to parameter settings
sub Format {
  croak 'Usage:  $html_string = My::KWIC::Format($lang, $start, $end [, $target, $keyword]);'
    unless @_ >= 3 and @_ <= 5;
  my $lang    = shift;
  my $start   = shift;
  my $end     = shift;
  my $target  = (@_) ? shift : -1;
  my $keyword = (@_) ? shift : -1;

  # display paramters: 'pos' only
  my $par_pos = My::Param::Get("pos");

  # load relevant attributes (to avoid multiple lookups)
  my $Word  = My::Config::GetAttribute( "word",  $lang );
  my $POS   = My::Config::GetAttribute( "pos",   $lang );
  my $Lemma = My::Config::GetAttribute( "lemma", $lang );

  # now go through range of cpos and format each token
  my @html = ();    # list of HTML-formatted tokens (and phrase boundaries)
  for my $cpos ( $start .. $end ) {
    my $html  = "";                                                          # process token annotations
    my $word  = ( $par_pos ne "lemmapos" ) ? $Word->cpos2str($cpos) : "";
    my $pos   = ( $par_pos ne "word" ) ? $POS->cpos2str($cpos) : "";
    my $lemma = ( $par_pos eq "lemmapos" ) ? $Lemma->cpos2str($cpos) : "";
    if ( $par_pos eq "word" ) {
      $html = escapeHTML($word);
    }
    elsif ( $par_pos eq "wordpos" ) {
      $html = escapeHTML("$word") . Sub( b( { -class => "blue narrow" }, escapeHTML("$pos") ) );
    }
    elsif ( $par_pos eq "lemmapos" ) {
      $html = escapeHTML("$lemma") . Sub( b( { -class => "blue narrow" }, escapeHTML("$pos") ) );
    }
    if ( $cpos == $keyword ) {
      $html = span( { -class => 'keyword' }, $html );
    }
    if ( $cpos == $target ) {
      $html = span( { -class => 'target' }, $html );
    }
    push @html, $html;
  }
  return "@html";
}

##
##  My::KWIC loaded successfully
##

return 1;
