package My::Query;
## -*-cperl-*-
## CQPDemo: wrapper around CQP and CEQL queries (simple POS tags are defined in Simple() function below)

use Carp;

use lib '.';
use My::Config;
use My::Param;

use CWB::CEQL;

# internal helper routine:
# $sort_clause = mk_sort_clause(); ... construct CQP sort clause based on attribute settings
sub mk_sort_clause {
  my $sort = My::Param::Get("sort");
  my $flags = (My::Param::Get("sort_fold")) ? '%cd' : "";
  my $attr = My::Param::Get("attribute");
  my $sort_clause = "";
  $sort_clause = "randomize 42"
    if $sort eq "random";
  $sort_clause = "by $attr $flags ascending"
    if $sort eq "asc";
  $sort_clause = "by $attr $flags descending"
    if $sort eq "desc";
  $sort_clause = "by $attr $flags descending reverse"
    if $sort eq "drev";
  $sort_clause = "by $attr $flags ascending reverse"
    if $sort eq "rev";
  return $sort_clause;
}

# ($id, $sort_clause) = My::Query::CQP();
#   execute CQP query, sort, and set keyword according to relevant parameter settings;
#   returns name of query result and optionally the CQP sort clause used
sub CQP {
  my $cqp = $My::Config::CQP;
  my $cache = $My::Config::Cache;
  my $corpus = $My::Config::Corpus;

  my $query = My::Param::Get("query");
  $query =~ s/\s+/ /g;                          # CQP doesn't take kindly to carriage returns, so normalise whitespace in query

  My::Config::Error("Please enter a query first.")
      if $query =~ /^\s*$/;

  my $subquery = My::Param::Get("subquery");
  $subquery =~ s/\s+/ /g;
  $subquery = "" if $subquery =~ /^\s*$/;

  # determine sort clause according to "sort" and "wordlemma" parameters
  my $sort_clause = mk_sort_clause();

  my $keyword = My::Param::Get("keyword");
  my $setk_cmd = "";
  if ($keyword) {
    my $strategy = My::Param::Get("keyword_strategy");
    my $condition = My::Param::Get("keyword_cond");
    my $direction = My::Param::Get("keyword_dir");
    my $size = My::Param::Get("keyword_size");
    my $base = My::Param::Get("keyword_base");
    $condition = "[ $condition ]"               # dwim brackets around keyword search pattern
      unless $condition =~ /^\s*[\[\"\']/;
    $setk_cmd = "$strategy $condition within $direction $size from $base";
    $setk_cmd .= " inclusive"
      if My::Param::Get("keyword_inclusive");
  }

  my $ID = $cache->query(-corpus => $corpus, 
                         -query => $query, -subquery => $subquery,
                         -sort => $sort_clause, -keyword => $setk_cmd, 
                         -cut => $My::Config::MaxMatches);
  if (wantarray) {
    return ($ID, $sort_clause);
  }
  else {
    return $ID;
  }
}


# ($id, $sort_clause) = My::Query::Simple();
#   execute simple query and sort results, according to relevant parameter settings;
#   the simple query expression is syntax-checked and converted into an equivalent CQP query;
#   returns name of query result
sub Simple {
  my $cqp = $My::Config::CQP;
  my $cache = $My::Config::Cache;
  my $corpus = $My::Config::Corpus;

  my $query = My::Param::Get("query");
  my $flags = My::Param::Get("casefold");
  my $language = lc(My::Param::Get("language"));

  My::Config::Error("Please enter a query first.")
      if $query =~ /^\s*$/;

  my $subquery = My::Param::Get("subquery");
  $subquery =~ s/\s+/ /g;
  $subquery = "" if $subquery =~ /^\s*$/;

  # define simple part-of-speech tags for each supported language
  my %tagset = (
    "en" => {
      "A" => "JJ.*",
      "ADJ" => "JJ.*",
      "N" => "N.*",
      "SUBST" => "N.*",
      "V" => "V.*|MD",
      "VERB" => "V.*|MD",
      "ADV" => "RB.*|WRB",
      "ART" => "P?DT",
      "CONJ" => "CC|IN",
      "INT" => "UH",
      "INTERJ" => "UH",
      "PREP" => "IN|TO",
      "PRON" => "P[RP].*|WP",
      '$' => "[.,:()\"`']|``|''|SENT",
      "STOP" => "[.]|SENT",
      "UNC" => "CD|EX|FW|LS|POS|RP|SYM|TO|WDT|WP.|.*[|].*"
    },
    "de" => {
      "A" => "ADJ.",
      "ADJ" => "ADJ.",
      "N" => "N.*",
      "SUBST" => "N.*",
      "V" => "V.*",
      "VERB" => "V.*",
      "ADV" => "ADV|ADJD|P.*AV",
      "ART" => "ART|APPRART|P.*AT",
      "CONJ" => "KO.*",
      "INT" => "ITJ",
      "INTERJ" => "ITJ",
      "PREP" => "AP.*",
      "PRON" => "P.*S|PRF",
      '$' => "[\$].*",
      "STOP" => "[\$].*",
      "UNC" => "CARD|FM|PTK.*|TRUNC|XY"      
    },
    "fr" => {
      "A" => "ADJ",
      "ADJ" => "ADJ",
      "N" => "N.*",
      "SUBST" => "N.*",
      "V" => "VER.*",
      "VERB" => "VER.*",
      "ADV" => "ADV.*",
      "ART" => "DET.*",
      "CONJ" => "CON.*",
      "INT" => "INT",
      "INTERJ" => "INT",
      "PREP" => "PRE.*",
      "PRON" => "PRO.*",
      '$' => "PON.*",
      "STOP" => "PON.*",
      "UNC" => "ABR|ADJ:.*|SYM"
    },
    "es" => {
      "A" => "ADJ",
      "ADJ" => "ADJ",
      "N" => "NC|NP|NM.*",
      "SUBST" => "NC|NP|NM.*",
      "V" => "V.*",
      "VERB" => "V.*",
      "ADV" => "ADV",
      "ART" => "ART|QU",
      "CONJ" => "(CC|CQUE|CSUB).*",
      "INT" => "ITJN",
      "INTERJ" => "ITJN",
      "PREP" => "PREP.*|PAL|PDEL",
      "PRON" => "PP.*|DM|INT|REL",
      '$' => "FS|CM|COLON|DASH|SEMICOLON",
      "STOP" => "FS|CM|COLON|DASH|SEMICOLON",
      "UNC" => "ACRNM|ALF.*|BACKSLASH|CARD|DOTS|LP|NEG|ORD|PE|PERCT|PNC|QUOT|RP|SE|SLASH|SYM|UMMX"
    },
    "it" => {
      "A" => "ADJ",
      "ADJ" => "ADJ",
      "N" => "NOM|NPR",
      "SUBST" => "NOM|NPR",
      "V" => "VER.*",
      "VERB" => "VER.*",
      "ADV" => "ADV",
      "ART" => "DET.*|PRE:det",
      "CONJ" => "CON",
      "INT" => "INT",
      "INTERJ" => "INT",
      "PREP" => "PRE.*",
      "PRON" => "PRO.*",
      '$' => "PON|SENT",
      "STOP" => "PON|SENT",
      "UNC" => "ABR|NUM|SYM",
    }
);
  
  # initialise CEQL parser
  my $parser = new CWB::CEQL;
  $parser->SetParam("simple_pos", $tagset{$language}); # regular expressions for simplified POS tags
  $parser->SetParam("s_attributes", {map {$_ => 1} qw(s p speaker chapter)});
  $parser->SetParam("default_ignore_case", ($flags =~ /c/) ? 1 : 0);
  $parser->SetParam("default_ignore_diac", ($flags =~ /d/) ? 1 : 0);
  
  # translate simple query and check for errors
  my $cqp_query = $parser->Parse($query);
  if (not defined $cqp_query) {
    My::Config::Error($parser->ErrorMessage);
  }

  # determine sort clause according to "sort" and "wordlemma" parameters
  my $sort_clause = mk_sort_clause();

  ## the following line can be useful for debugging
  # print STDERR "SIMPLE QUERY: $cqp_query\n";

  # execute CQP query and return result
  my $ID = $cache->query(-corpus => $corpus, 
                         -query => $cqp_query, -subquery => $subquery, -sort => $sort_clause,
                         -cut => $My::Config::MaxMatches);
  if (wantarray) {
    return ($ID, $sort_clause, $cqp_query);
  }
  else {
    return $ID;
  }
}


# $id = My::Query::Auto();
#   execute CQP or simple query (according to "mode" parameter) and return name of query result
sub Auto {
  my $mode = My::Param::Get("mode");
  if ($mode eq "cqp") {
    return CQP();
  }
  else {
    return Simple();
  }
}


##
##  My::Query loaded successfully
##

return 1;
