#!/usr/bin/perl -w
## -*-cperl-*-
## CQPDemo: word frequency lists and automatic generation of translation candidates

use lib '.';
use My::Config;
use My::Param;
use My::Forms;
use My::Query;

use CWB;
use CWB::CEQL;

use CGI qw/:standard *table *p/;

$max_translate_freq = 50000;    # hard limits on frequency of words that can be translated
$min_translate_freq = 10;
$max_cand           = 42;       # maximal number of translation candidates (per language)
$min_Dice           = .001;     # lower bound on Dice score (don't show candidates with lower scores)

$corpus = $My::Config::Corpus;

$tool          = My::Param::Get("tool");
$attribute     = My::Param::Get("attribute");
$wlfilter      = My::Param::Get("wlfilter");
$wlfilter_fold = My::Param::Get("casefold");
$cutoff        = My::Param::Get("wlcutoff");
$sort          = My::Param::Get("wlsort");
$wlfold        = My::Param::Get("wlfold");
$main_language = My::Param::Get("language");
$corpus_handle = $My::Config::CorpusHandle{$main_language}; # for case/accent folding
$main_language = lc($main_language); # make sure that language code is lowercase (assumed by code below)

if ( $tool eq "wordlist" ) {
  ## generate word list for selected language
  $tf = new CWB::TempFile "CQPDemo.gz";
  $tf->finish;
  $tempfile = $tf->name;
  $filter_flag = "";
  if ($wlfilter ne "*") {
    my $parser = new CWB::CEQL; # use CEQL wildcard patterns
    my $cqp_regexp = $parser->Parse($wlfilter, "wildcard_pattern");
    if (not defined $cqp_regexp) {
      My::Config::Error("Invalid wildcard pattern '$wlfilter':", $parser->ErrorMessage);
    }
    $cqp_regexp =~ s/^"//; $cqp_regexp =~ s/"$//; # translate double-quoted CQP regexp into command-line form
    $cqp_regexp =~ s/'/'"'"'/g; # this is the safe way to escape apostrophes in bash
    $filter_flag = "-O -p '$cqp_regexp'";
  }
  $filter_flag .= " -$wlfilter_fold"
    unless ( $wlfilter_fold eq "" ) or ( $filter_flag eq "" );
  $cmd = "$CWB::Lexdecode $My::Config::RegistryFlag -f -P $attribute $filter_flag $corpus | gzip > $tempfile";
  CWB::Shell::Cmd($cmd);

  # read data from temporary file into word list + frequency hash
  @words = ();
  %freq  = ();
  while ( $line = $tf->read ) {
    chomp $line;
    $line =~ /^\s*([0-9]+)\s+(\S.*?)\s*$/
      or die "Internal error parsing output of cwb-lexdecode or cwb-scan-corpus.\nOffending line: ``$line''";
    $f    = $1;
    $word = $2;
    $word =~ s/[\x00-\x20]+/ /g;    # normalise whitespace and control characters
    next if $f < $cutoff;
    $word = $corpus_handle->normalize($wlfold, $word)
      if $wlfold ne "";
    $freq{$word} += $f;
  }
  $tf->close;

  # sort word list according to specification
  @words = keys %freq;
  if ( $sort eq "freq" ) {          # freq -> sort by descending frequency
    my %sort_key = map { $_ => $corpus_handle->normalize("cd", $_) } @words; # pre-compute folded strings for lexical sorting
    @words = sort { $freq{$b} <=> $freq{$a} or $sort_key{$a} cmp $sort_key{$b} or $a cmp $b } @words;
  }
  elsif ( $sort eq "rev" ) {        # rev -> sort reverse (from end of string)
    @words = map { scalar reverse $_ } @words;
    my %sort_key = map { $_ => $corpus_handle->normalize("cd", $_) } @words; # pre-compute folded strings (reverse) for lexical sorting
    @words = sort { $sort_key{$a} cmp $sort_key{$b} or $a cmp $b } @words;
    @words = map { scalar reverse $_ } @words;
  }
  else {                            # lexical -> sort lexically (depends on current locale)
    my %sort_key = map { $_ => $corpus_handle->normalize("cd", $_) } @words; # pre-compute folded strings for lexical sorting
    @words = sort { $sort_key{$a} cmp $sort_key{$b} or $a cmp $b } @words;
  }

  # finally, prepare HTML page
  print header,
    start_html(
    -title => "CQPDemo: Word List",
    -style => { -src => "$My::Config::CSSDir/standard.css" }
    ),
    "\n";
  print p( "Click on word to display examples, click on ", b( { -class => "orange" }, "T" ), " for translation candidates (not available for all words)." );

  if ( $sort eq "rev" ) {
    print start_table,    # layout for reverse sort:  word, T, f
      Tr(
      th( { -width => 150 },     '&nbsp;' ),     # indentation
      th( { -align => "right" }, $attribute ),
      th( { -width => 15, -align => "center" }, '&nbsp;' ),
      th( { -align => "left" }, "f" )
      ),
      "\n";
  }
  else {
    print start_table,                           # standard layout:  f, T, word
      Tr(
      th( { -width => 150 },     '&nbsp;' ),     # indentation
      th( { -align => "right" }, "f" ),
      th( { -width => 15, -align => "center" }, '&nbsp;' ),
      th( { -align => "left" }, $attribute )
      ),
      "\n";
  }

  foreach $word (@words) {
    $wordlink = escapeHTML($word);               # HTML encoding of $word with link to display of instances
    ## construct CQP query for looking up word usage examples
    $flags          = ($wlfold) ? "\%$wlfold" : "";
    $query          = "[ $attribute = '" . quotemeta($word) . "' $flags]";
    $url            = "display.pl?" . My::Forms::MakeGETString( [qw(language context pos show_*)], { mode => "cqp", query => $query } );
    $wordlink       = a( { -href => $url, -target => "context" }, $wordlink );
    $translate_url  = "wordlist.pl?" . My::Forms::MakeGETString( [qw(language attribute show_*)], { tool => "translate", mode => "cqp", query => "$query" } );
    $translate_link = a( { -href => $translate_url, -target => "context" }, b( { -class => "orange" }, "T" ) );
    $translate_link = ""
      if $freq{$word} < $min_translate_freq
        or $freq{$word} > $max_translate_freq;

    if ( $sort eq "rev" ) {
      print Tr( td(), td( { -align => "right", -class => "serif" }, $wordlink ), td( { -align => "center" }, $translate_link ), td( { -align => "left" }, $freq{$word} ), ), "\n";
    }
    else {
      print Tr( td(), td( { -align => "right" }, $freq{$word} ), td( { -align => "center" }, $translate_link ), td( { -align => "left", -class => "serif" }, $wordlink ), ), "\n";
    }
  }
  print end_table, "\n";

  print end_html, "\n";
}
elsif ( $tool eq "translate" ) {
  $cqp = $My::Config::CQP;
  $id  = My::Query::Auto();
  ($size) = $cqp->exec("size $id");
  @cpos = $cqp->exec("tabulate $id match");

  print header,
    start_html(
    -title => "CQPDemo: Translation candidates",
    -style => { -src => "$My::Config::CSSDir/standard.css" }
    ),
    "\n",
    h2( "Translation candidates for: ", span( { -class => "red" }, escapeHTML( My::Param::Get("query") ) ), " ($size matches)" ),
    "\n";

  @languages = grep { $_ ne $main_language and My::Param::Get("show_$_") } map {lc} @My::Config::Languages;
  foreach $lang (@languages) {
    print h3( $My::Config::LanguageNames{uc($lang)} ), "\n";
    $target_corpus = $My::Config::CorpusBase . uc($lang);
    $align_att     = My::Config::GetAttribute( lc($target_corpus) );

    ## determine aligned regions in target corpus
    $regions = new CWB::TempFile "cqpdemo_translate.tbl";
    $max_alg = -1;                                          # make sure that we process each alignment region only once, even if there are multiple matches
    foreach $cpos (@cpos) {
      $alg = $align_att->cpos2alg($cpos);
      next if ( not defined $alg ) or $alg <= $max_alg;
      $max_alg = $alg;
      ( undef, undef, $t_start, $t_end ) = $align_att->alg2cpos($alg);
      $regions->write("$t_start\t$t_end\n");
    }
    $regions->finish;
    $regions_file = $regions->name;

    ## now scan aligned regions for candidate translations
    $candidates = new CWB::TempFile "cqpdemo_translate.gz";
    $candidates->finish;
    $candidates_file = $candidates->name;
    $scan_cmd        = "$CWB::ScanCorpus -R '$regions_file' -f $min_translate_freq -o '$candidates_file' $target_corpus $attribute";
    CWB::Shell::Cmd($scan_cmd);
    $regions->close;

    ## read candidates and calculate (approximate) Dice scores
    $target_att = My::Config::GetAttribute( $attribute, $lang );
    %Score = ();    # $Score{$cand} = $score;
    while ( $line = $candidates->read ) {
      chomp($line);
      ( $f, $cand ) = split /\t/, $line;
      $target_id = $target_att->str2id($cand);
      next unless defined $target_id;    # this shouldn't happen!
      $target_size  = $target_att->id2freq($target_id);
      $Dice         = 2 * $f / ( $size + $target_size );    # very rough Dice approximation
      $Score{$cand} = $Dice;
    }
    $candidates->close;

    ## show first <n> candidates ranked by Dice scores
    @cand_list =
      sort { $Score{$b} <=> $Score{$a} or $a cmp $b }
      grep { $Score{$_} >= $min_Dice } keys %Score;
    @cand_list = @cand_list[ 0 .. $max_cand - 1 ]
      if @cand_list > $max_cand;
    print start_p( { -class => "serif" } );
    foreach $cand (@cand_list) {
      $score = sprintf "%.4f", $Score{$cand};    # format with 4 decimal digits, no leading 0
      $score =~ s/^0\././;
      print escapeHTML($cand), Sub( span( { -class => "blue sans" }, $score ) ), ", ";
    }
    print '&hellip;', end_p, "\n";
  }

  print end_html, "\n";
}
else {
  die "Internal error: no handler for corpus tool '$tool'.";
}
